<?php

namespace Bundler {

	// use stdClass;
	use Bundler\Controllers\Integrations;
	use Bundler\Helpers\Ajax;
	use Bundler\Helpers\Cart;
	use Bundler\Models\Settings;
	use Bundler\Models\DB;
	use Bundler\Views\Widget;

	/**
	 * Main WBundler class.
	 *
	 * @since 1.0.0
	 */
	final class Bundler
	{

		/**
		 * Bundler Instance
		 *
		 * @since 1.0.0
		 *
		 */
		private static $instance;

		protected $cart_helper;

		/**
		 * Plugin version for enqueueing, etc.
		 * The value is got from WBDL_VERSION constant.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $version = '';

		/**
		 * Plugin settings.
		 *
		 * @since 1.0.0
		 *
		 * @var Settings
		 */
		private $settings;

		/**
		 * Plugin License.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		private $license;

		/**
		 * Pro returns true, free (Lite) returns false.
		 *
		 * @since 1.0.0
		 *
		 * @var bool
		 */
		private $pro = false;


		/**
		 * Main WBundler Instance.
		 *
		 * Only one instance of Bundler exists in memory at any one time.
		 * Also prevent the need to define globals all over the place.
		 *
		 * @since 1.0.0
		 *
		 * @return Bundler
		 */
		public static function instance()
		{

			if (
				self::$instance === null ||
				!self::$instance instanceof self
			) {

				self::$instance = new self();
				self::$instance->constants();
				self::$instance->includes();
				self::$instance->hooks();

				add_action('init', [self::$instance, 'load_textdomain'], 10);
				add_action('plugins_loaded', [self::$instance, 'objects'], 10);
			}

			return self::$instance;
		}

		/**
		 * Register hooks.
		 *
		 * @since 1.0.0
		 */
		public function hooks()
		{

			// Create/Update Database
			add_action('admin_init', function () {
				DB::create_tables();
			});

			// Enqueue scripts
			add_action('admin_enqueue_scripts', [$this, 'enqueue_admin_style']);
			add_action('admin_enqueue_scripts', [$this, 'enqueue_admin_script']);
			add_action('wp_enqueue_scripts', [$this, 'enqueue_script']);
			add_action('wp_enqueue_scripts', [$this, 'enqueue_style']);

			// Add the settings page link to plugins page
			add_filter('plugin_action_links_' . WBDL_PLUGIN_BASENAME, function ($links) {
				$reordered_links = array(
					'Get Bundler Pro' => '<a target="_blank" href="https://wcbundler.com/pricing" style="color:#93003c;font-weight:700;">' . __('Get Bundler Pro') . '</a>',
					'settings' => '<a href="' .
						admin_url('admin.php?page=bundler/settings') .
						'">' . __('Settings') . '</a>',
					'docs' => '<a target="_blank" href="https://wcbundler.com/docs">' . __('Docs') . '</a>',
					'support' => '<a target="_blank" href="https://wcbundler.com/support">' . __('Support') . '</a>',

				);

				$links = array_merge($reordered_links, $links);
				return $links;
			});

			// Init tracker
			add_action('plugins_loaded', [$this, 'init_tracker']);
		}

		/**
		 * Enqueue Scripts.
		 *
		 * @since 1.0.0
		 */
		public function enqueue_script()
		{
			wp_register_script('bundler_js', WBDL_PLUGIN_URL . 'assets/js/app.js', array('jquery'), '2.4.1');
			wp_enqueue_script('bundler_js');

			if ($this->settings && is_array($this->settings)) {
				$cart_redirect = $this->settings[0]->cart_redirect;
			}

			wp_localize_script(
				'bundler_js',
				'script_vars',
				array(
					'woo_cart_url'  => wc_get_cart_url(),
					'cart_redirect' => $cart_redirect ? $cart_redirect : 'off',
					'ajax_url'      => admin_url('admin-ajax.php'),
					'wc_endpoints'  => AJAX::get_public_endpoints(),
				)
			);
		}

		/**
		 * Enqueue Styles.
		 *
		 * @since 1.3.5
		 */
		public function enqueue_style()
		{
			wp_register_style('bundler_style', WBDL_PLUGIN_URL . 'assets/css/style.css', false, WBDL_VERSION, 'all');
			wp_enqueue_style('bundler_style');
		}

		/**
		 * Enqueue Admin JS.
		 *
		 * @since 1.3.6
		 */
		public function enqueue_admin_script()
		{
			wp_enqueue_script('bundler_admin_js', WBDL_PLUGIN_URL . 'assets/admin/js/admin.js', array('jquery'), '2.5.1');
			wp_enqueue_script('bundler_admin_coloris_js', WBDL_PLUGIN_URL . 'assets/admin/js/coloris.min.js', array('jquery'), '2.5.1');
			wp_enqueue_script('selectWoo', WBDL_PLUGIN_URL . 'assets/admin/js/selectWoo.full.js', array('jquery'), '1.0.0', true);
			wp_localize_script(
				'bundler_admin_js',
				'admin_script_vars',
				array(
					'ajax_url' => admin_url('admin-ajax.php'),
					'offers_page_url' => admin_url('admin.php?page=bundler')
				)
			);
		}

		/**
		 * Enqueue Admin CSS styles.
		 *
		 * @since 1.0.0
		 */
		public function enqueue_admin_style()
		{
			wp_enqueue_style('admin-settings', WBDL_PLUGIN_URL . 'assets/admin/css/admin-settings.css', false, WBDL_VERSION, 'all');
			wp_enqueue_style('admin-offers', WBDL_PLUGIN_URL . 'assets/admin/css/admin-offers.css', false, WBDL_VERSION, 'all');
			wp_enqueue_style('admin-coloris', WBDL_PLUGIN_URL . 'assets/admin/css/coloris.min.css', false, WBDL_VERSION, 'all');
			wp_enqueue_style('selectWoo', WBDL_PLUGIN_URL . 'assets/admin/css/selectWoo.css');
			wp_enqueue_style('icofont', WBDL_PLUGIN_URL . 'assets/admin/css/icofont/icofont.css', false, WBDL_VERSION, 'all');
		}

		/**
		 * Setup plugin constants.
		 * All the path/URL related constants are defined in main plugin file.
		 *
		 * @since 1.0.0
		 */
		private function constants()
		{

			$this->version = WBDL_VERSION;

			// Plugin Slug - Determine plugin type and set slug accordingly.

			if (WBDL_PRO == true) {
				$this->pro = true;
				define('WBDL_PLUGIN_SLUG', 'bundler-pro');
			} else {
				$this->pro = false;
				define('WBDL_PLUGIN_SLUG', 'bundler-lite');
			}
		}


		/**
		 * Load the plugin language files.
		 *
		 * @since 1.0.0
		 * 
		 */
		public function load_textdomain()
		{

			// If the user is logged in, unset the current text-domains before loading our text domain.
			// This feels hacky, but this way a user's set language in their profile will be used,
			// rather than the site-specific language.
			if (is_user_logged_in()) {
				unload_textdomain('woobundles');
			}

			load_plugin_textdomain('woobundles', false, dirname(plugin_basename(WBDL_PLUGIN_FILE)) . '/assets/languages/');
		}

		/**
		 * Include files.
		 *
		 * @since 1.0.0
		 */
		private function includes()
		{

			// Load DB
			require_once WBDL_PLUGIN_DIR . 'App/Models/DB.php';

			// Include models
			require_once WBDL_PLUGIN_DIR . 'App/Models/Bundle.php';
			require_once WBDL_PLUGIN_DIR . 'App/Models/Offer.php';
			require_once WBDL_PLUGIN_DIR . 'App/Models/Settings.php';

			// Include controllers
			require_once WBDL_PLUGIN_DIR . 'App/Controllers/OfferController.php';
			require_once WBDL_PLUGIN_DIR . 'App/Controllers/SettingsController.php';
			require_once WBDL_PLUGIN_DIR . 'App/Controllers/Integrations.php';

			// Include helpers
			require_once WBDL_PLUGIN_DIR . 'App/Helpers/Ajax.php';
			require_once WBDL_PLUGIN_DIR . 'App/Helpers/Cart.php';

			// The widget
			require_once WBDL_PLUGIN_DIR . 'App/Views/Widget.php';

			// Admin/Dashboard only includes, also in ajax.
			if (is_admin()) {
				// Load the plugin's settings section.
				require_once WBDL_PLUGIN_DIR . 'App/Views/admin/AdminApp.php';
			}
		}


		/**
		 * Setup objects.
		 *
		 * @since 1.0.0
		 */
		public function objects()
		{
			// Global objects.
			$this->cart_helper = new Cart();
			$this->settings = Settings::get_settings();

			Widget::init();
			Ajax::init();
			Integrations::init();

			// Hook now that all of the WBundler stuff is loaded.
			do_action('wbdl_loaded');
		}

		/**
		 * Initialize the plugin tracker
		 *
		 * @return void
		 */
		function init_tracker()
		{
			if (!class_exists('Appsero\Client')) {
				require_once WBDL_PLUGIN_DIR . '/includes/appsero/src/Client.php';
			}
			$client = new \Appsero\Client('86d8a14f-02c3-4b05-9c4f-5839e6130e96', 'Bundler Lite', WBDL_PLUGIN_FILE);
			$client->insights()->init();
		}

		/**
		 * Whether the current instance of the plugin is a paid version, or free.
		 *
		 * @since 1.0.0
		 *
		 * @return bool
		 */
		public function is_pro()
		{
			/**
			 * Filters whether the current plugin version is pro.
			 *
			 * @since 1.0.0
			 *
			 * @param bool $pro Whether the current plugin version is pro.
			 */
			return (bool) $this->pro;
		}
	}
}

namespace {

	/**
	 * The function which returns the one WBundler instance.
	 *
	 * @since 1.0.0
	 *
	 * @return Bundler\Bundler
	 */
	function wbdl()
	{
		return Bundler\Bundler::instance();
	}

	/**
	 * Adding an alias for backward-compatibility with plugins
	 * that still use class_exists( 'WBundler' )
	 * instead of function_exists( 'wbdl' ), which is preferred.
	 *
	 * @since 1.0.0
	 */
	class_alias('Bundler\Bundler', 'Bundler');
}
