<?php

namespace Bundler\Controllers;

use Bundler\Models\Offer;
use Bundler\Models\Bundle;

if (!defined('ABSPATH')) exit;

class OfferController
{
    public function __construct()
    {
    }

    /**
     * Get all offers
     * 
     * @return array Bundler\Models|Offer
     */
    public function get_all_offers()
    {
        return Offer::get_all_offers();
    }

    /**
     * Get an offer by id
     * 
     * @param mixed $offer_id
     * 
     * @return Bundler\Models|Offer
     */
    public function get_offer_by_id($offer_id)
    {
        return Offer::get_offer_by_id($offer_id);
    }

    /**
     * Get bundles for a given offer
     * 
     * @param mixed $offer_id
     * 
     * @return Bundler\Models\Bundle
     */
    public function get_bundles_by_offer_id($offer_id)
    {
        return Bundle::get_bundles_by_offer_id($offer_id);
    }

    /**
     * Get the highest priority offer for the product id
     * 
     * @param mixed $product_id
     * 
     * @return Bundler\Models\Offer
     */
    public function get_the_offer($product_id)
    {
        return Offer::get_the_offer($product_id);
    }

    /**
     * Get the bundles of the active offer that has the higher priority for a certain product
     * 
     * @param mixed $product_id
     * 
     * @return array|null
     */
    public function get_the_bundles($product_id)
    {
        $offer =  Offer::get_the_offer($product_id);
        if ($offer) {
            $offer_id = $offer->id;
            return Bundle::get_bundles_by_offer_id($offer_id);
        }

        return null;
    }

    /**
     * Get bundles by product ID, sorted by Qty Desc. This is used to calculate the discount in the cart.
     * 
     * @param mixed $product_id
     * 
     * @return array|null
     */
    public function get_the_bundles_desc($product_id)
    {
        $offer =  Offer::get_the_offer($product_id);
        if ($offer) {
            $offer_id = $offer->id;
            return Bundle::get_bundles_by_offer_id_sort_by_qty_desc($offer_id);
        }

        return null;
    }

    /**
     * Remove offer by offer id
     * 
     * @param mixed $offer_id
     * 
     * @return void
     */
    public function remove_offer($offer_id)
    {
        Bundle::remove_bundles($offer_id);
        Offer::remove_offer($offer_id);
    }

    /**
     * Add new offer
     * 
     * @param @product_id
     * @param @offer_data
     * 
     * @return void
     */
    public function new_offer($product_id, $offer_data)
    {
        if ($product_id && $offer_data) {

            $title         = sanitize_text_field($offer_data['offer_title']);
            $discount_type = isset($offer_data['discount_type']) ? sanitize_text_field($offer_data['discount_type']) : 'bundle';
            $status        = isset($offer_data['offer_status']) ? 'on' : 'off';
            $priority      = isset($offer_data['priority']) ? sanitize_text_field($offer_data['priority']) : sanitize_text_field($offer_data['offer_id']);
            $count         = count($offer_data['bundle_title']);

            $field_names = array(
                'bundle_title',
                'image_url',
                'product_qty',
                'sale_price',
                'regular_price',
                'option1_name',
                'option1_value',
                'option2_name',
                'option2_value',
                'hidden_add_messages',
                'message',
                'discount_rule',
                'message_effect',
            );

            $inputs = array();

            for ($i = 0; $i < $count; $i++) {

                if ($offer_data['bundle_title'][$i] != '') :

                    foreach ($field_names as $field) {
                        $input_value = isset($offer_data[$field][$i]) ? $offer_data[$field][$i] : '';
                        $inputs[$field][$i] = sanitize_text_field(stripslashes(strip_tags($input_value)));
                    }

                    $inputs['ppu'][$i]                   = 'off';
                    $inputs['use_custom_variations'][$i] = 'off';

                    if (($offer_data['preselected_offer_id'] != "off") && ($i == $offer_data['preselected_offer_id'])) {
                        $inputs['preselected_offer'][$i] = 'on';
                    } else {
                        $inputs['preselected_offer'][$i] = 'off';
                    }

                endif;
            }

            $offer_id = Offer::new_offer($product_id, $title, $discount_type, $status, $priority);

            for ($i = 0; $i < $count; $i++) {
                if ($inputs['bundle_title'][$i] != '') {
                    Bundle::add_bundle_to_offer(
                        $offer_id,
                        $inputs['bundle_title'][$i],
                        $inputs['image_url'][$i],
                        $inputs['product_qty'][$i],
                        $inputs['sale_price'][$i],
                        $inputs['regular_price'][$i],
                        $inputs['use_custom_variations'][$i],
                        $inputs['option1_name'][$i],
                        $inputs['option1_value'][$i],
                        $inputs['option2_name'][$i],
                        $inputs['option2_value'][$i],
                        $inputs['hidden_add_messages'][$i],
                        $inputs['message'][$i],
                        $inputs['discount_rule'][$i],
                        $inputs['preselected_offer'][$i],
                        $inputs['message_effect'][$i],
                        $inputs['wmc_sale_price'][$i],
                        $inputs['wmc_regular_price'][$i],
                        $inputs['ppu'][$i]
                    );
                }
            }
        }
    }

    /**
     * update an offer
     * 
     * @param $offer_id
     * @param $product_id
     * @param @offer_data
     * 
     * @return void
     */
    public function update_offer($offer_id, $product_id, $offer_data)
    {
        if ($offer_id && $product_id) {

            $title         = sanitize_text_field($offer_data['offer_title']);
            $discount_type = isset($offer_data['discount_type']) ? sanitize_text_field($offer_data['discount_type']) : 'bundle';
            $status        = isset($offer_data['offer_status']) ? 'on' : 'off';
            $priority      = isset($offer_data['priority']) ? sanitize_text_field($offer_data['priority']) : sanitize_text_field($offer_data['offer_id']);
            $count         = count($offer_data['bundle_title']);

            $field_names = array(
                'bundle_title',
                'image_url',
                'product_qty',
                'sale_price',
                'regular_price',
                'option1_name',
                'option1_value',
                'option2_name',
                'option2_value',
                'hidden_add_messages',
                'message',
                'discount_rule',
                'message_effect',
            );

            $inputs = array();

            for ($i = 0; $i < $count; $i++) {

                if ($offer_data['bundle_title'][$i] != '') :

                    foreach ($field_names as $field) {
                        $input_value = isset($offer_data[$field][$i]) ? $offer_data[$field][$i] : '';
                        $inputs[$field][$i] = sanitize_text_field(stripslashes(strip_tags($input_value)));
                    }

                    $inputs['ppu'][$i]                   = 'off';
                    $inputs['use_custom_variations'][$i] = 'off';
                    if (($offer_data['preselected_offer_id'] != "off") && ($i == $offer_data['preselected_offer_id'])) {
                        $inputs['preselected_offer'][$i] = 'on';
                    } else $inputs['preselected_offer'][$i] = 'off';

                endif;
            }

            Offer::update_offer($offer_id, $product_id, $title, $discount_type, $status, $priority);

            Bundle::remove_bundles($offer_id);

            for ($i = 0; $i < $count; $i++) {
                if ($inputs['bundle_title'][$i] != '') {
                    Bundle::add_bundle_to_offer(
                        $offer_id,
                        $inputs['bundle_title'][$i],
                        $inputs['image_url'][$i],
                        $inputs['product_qty'][$i],
                        $inputs['sale_price'][$i],
                        $inputs['regular_price'][$i],
                        $inputs['wmc_sale_price'][$i],
                        $inputs['wmc_regular_price'][$i],
                        $inputs['use_custom_variations'][$i],
                        $inputs['option1_name'][$i],
                        $inputs['option1_value'][$i],
                        $inputs['option2_name'][$i],
                        $inputs['option2_value'][$i],
                        $inputs['hidden_add_messages'][$i],
                        $inputs['message'][$i],
                        $inputs['discount_rule'][$i],
                        $inputs['preselected_offer'][$i],
                        $inputs['message_effect'][$i],
                        $inputs['ppu'][$i],
                    );
                }
            }
        }
    }

    /**
     * update an offer status (enable or disable)
     * 
     * @param mixed $offer_id
     * @param mixed $status
     * 
     * @return void
     */
    public function update_offer_status($offer_id, $status)
    {
        return Offer::update_offer_status($offer_id, $status);
    }
}
