<?php

/**
 * AJAX
 *
 * @since 1.0.0
 * 
 */

namespace Bundler\Helpers;

use Bundler\Controllers\OfferController;

if (!defined('ABSPATH')) exit;

class Ajax
{

    /**
     * Hook in ajax handlers.
     * 
     * @return void
     */
    public static function init()
    {
        self::handle_public_ajax();
        self::handle_private_ajax();
    }

    /**
     * Add public ajax endpoints
     *
     * @return void
     */
    public static function handle_public_ajax()
    {
        $endpoints = self::get_available_public_endpoints();
        foreach ($endpoints as $action => $function) {
            add_action('wc_ajax_' . $action, array(__CLASS__, $function));
        }
    }

    /**
     * Add private ajax endpoints
     *
     * @return void
     */
    public static function handle_private_ajax()
    {
        $endpoints = self::get_available_private_endpoints();
        foreach ($endpoints as $action => $function) {
            add_action('wp_ajax_' . $action, array(__CLASS__, $function));
            add_action('wp_ajax_nopriv_' . $action, array(__CLASS__, $function));
        }
    }

    /**
     * Get WC public endpoints
     *
     * @param $query
     *
     * @return array
     */
    public static function get_public_endpoints($query = [])
    {
        $public_endpoints = self::get_available_public_endpoints();
        if (empty($public_endpoints) || !is_array($public_endpoints)) {
            return [];
        }

        $endpoints = [];
        foreach ($public_endpoints as $key => $function) {
            $url = \WC_AJAX::get_endpoint($key);
            $url = is_array($query) && count($query) > 0 ? add_query_arg($query, $url) : $url;

            $endpoints[$key] = $url;
        }

        return $endpoints;
    }

    /**
     * Get public endpoints     
     *
     * @return array
     */
    public static function get_available_public_endpoints()
    {
        return [
            'wbdl_add_to_cart'            => 'add_to_cart',
            'wbdl_add_to_cart_custom_var' => 'add_to_cart_custom_var',
        ];
    }

    /**
     * Get private endpoints
     *
     * @return array
     */
    public static function get_available_private_endpoints()
    {
        return [
            'wbdl_search_products'     => 'search_products',
            'wbdl_new_offer'           => 'new_offer',
            'wbdl_update_offer'        => 'update_offer',
            'wbdl_delete_offer'        => 'delete_offer',
            'wbdl_change_offer_status' => 'change_offer_status',
            'wbdl_update_price'        => 'update_price',
        ];
    }

    /**
     * Add to cart function for simple and variable products
     * 
     * @return void
     */
    public static function add_to_cart()
    {

        self::verify_nonce();

        $product_id        = apply_filters('woocommerce_add_to_cart_product_id', absint(sanitize_text_field($_POST['wbdl_product_id'])));
        $products_number   = empty($_POST['wbdl_products_num']) ? 1 : wc_stock_amount(sanitize_text_field($_POST['wbdl_products_num']));
        $qty               = isset($_POST['wbdl_product_quantity']) ? sanitize_text_field($_POST['wbdl_product_quantity']) : 1;
        $product_status    = get_post_status($product_id);
        $passed_validation = true;

        $product = wc_get_product($product_id);
        $cart_item_meta = array();

        if ($product->is_type('simple')) {

            $cart_item_meta['_wbdl_data'] = ['use_custom_variations' => 'off'];

            $qty = $qty * $products_number;

            if ($product) $stock_quantity = $product->get_stock_quantity();
            if (!self::is_product_in_stock($product_id, $qty)) {
                $passed_validation = false;
            }

            if ($passed_validation && 'publish' === $product_status && WC()->cart->add_to_cart($product_id, $qty, 0, null, $cart_item_meta)) {
                do_action('woocommerce_ajax_added_to_cart', $product_id);
                if ('yes' === get_option('woocommerce_cart_redirect_after_add')) {
                    wc_add_to_cart_message(array($product_id => $qty), true);
                }
                \WC_AJAX::get_refreshed_fragments();
            } else {
                $response = array(
                    'error'       => true,                                                                                            // You cannot add that amount to the cart — we have 1 in stock and you already have 1 in your cart.
                    'message'     => 'This product has only ' . $stock_quantity . ' items in stock.',
                    'product_url' => apply_filters('woocommerce_cart_redirect_after_error', get_permalink($product_id), $product_id)
                );
                wp_send_json($response);
            }
        }
        wp_die();
    }

    /**
     * ajax product search
     * 
     * @return void
     */
    public static function search_products()
    {
        self::verify_nonce($_REQUEST['wbdl_nonce']);

        //For loading all language Product in select box.
        //$this->load_all_wpml_products();
        //$query = $this->input->post('query', '');
        $query = isset($_POST['query']) ? sanitize_text_field($_POST['query']) : '';
        if ($query) {
            // Disable other search classes
            remove_all_filters('woocommerce_data_stores');
            // Search for products with the specified query
            $product_query = array(
                'limit' => 10,
                'status' => 'publish',
                's' => $query,
            );
            if (!WBDL_PRO) $product_query += array(
                'type' => 'simple'
            );
            $products = wc_get_products($product_query);
            // Format the results
            $formatted_products = array_map(function ($product) {
                return array(
                    'id'   => (string) $product->get_id(),
                    'text' => '#' . $product->get_id() . ' ' . $product->get_title(),
                );
            }, $products);

            wp_send_json_success($formatted_products);
        }

        wp_send_json_success([]);
    }

    /**
     * AJAX new offer
     * 
     * @return json
     */
    public static function new_offer()
    {
        // Parse the data
        $offer_data = array();
        parse_str($_POST['data'], $offer_data);

        self::verify_nonce($offer_data['wbdl_nonce']);

        $product_id = $offer_data['product_id'];
        $product_type = wc_get_product($product_id)->get_type();

        if (!WBDL_PRO && ($product_type != 'simple')) return wp_send_json('You are on Bundler Lite. You cannot create offers for variable products.');

        // Get the data and create the new offer
        $offer_controller = new OfferController();
        $offer_controller->new_offer($product_id, $offer_data);

        // Ajax response
        return wp_send_json('success');
    }

    /**
     * AJAX update offer
     * 
     * @return json
     */
    public static function update_offer()
    {
        // Parse the data
        $offer_data = array();
        parse_str($_POST['data'], $offer_data);

        self::verify_nonce($offer_data['wbdl_nonce']);

        $product_id = $offer_data['product_id'];
        $product_type = wc_get_product($product_id)->get_type();

        if (!WBDL_PRO && ($product_type != 'simple')) return wp_send_json('You are on Bundler Lite. You cannot edit offers for variable products.');

        // Get the data and update the offer
        $offer_id = $offer_data['offer_id'];

        $offer_controller = new OfferController();
        $offer_controller->update_offer($offer_id, $product_id, $offer_data);

        return wp_send_json('success');
    }

    /**
     * AJAX delete offer
     * 
     * @return json
     */
    public static function delete_offer()
    {
        // Get the data and update the offer
        $offer_id = $_POST['offer_id'];

        $offer_controller = new OfferController();
        $offer_controller->remove_offer($offer_id);

        // Ajax response
        return wp_send_json('success');
    }

    /**
     * AJAX change offer status (enable or disble)
     * 
     * @return json
     */
    public static function change_offer_status()
    {
        // Get the data and update the offer
        $offer_id = $_POST['offer_id'];
        $status = $_POST['status'];
        $offer_controller = new OfferController();
        $offer_controller->update_offer_status($offer_id, $status);

        return wp_send_json('success');
    }

    /**
     * Verify nonce
     *
     * @return void
     */
    public static function verify_nonce($form_nonce = null)
    {
        $nonce = $form_nonce ? $form_nonce : filter_input(INPUT_POST, 'nonce', FILTER_UNSAFE_RAW);
        if (is_null($nonce) || !wp_verify_nonce($nonce, 'bundler')) {
            wp_send_json(array(
                'msg'  => __('Security check failed', 'woobundles'),
                'code' => 401
            ));
        }
    }

    /**
     * @param mixed $product_id
     * @param mixed $qty
     * 
     * @return bool
     */
    private static function is_product_in_stock($product_id, $qty)
    {
        $product          = wc_get_product($product_id);
        $quantity_in_cart = 0;

        if (WC()->cart) {
            $cart = WC()->cart->cart_contents;
        }
        if ($cart) {
            foreach ($cart as $cart_item) {
                if ($cart_item['product_id'] == $product_id) {
                    $quantity_in_cart++;
                }
            }
        }
        $total_quantity = $qty + $quantity_in_cart;

        return $product && $product->is_in_stock() && $product->has_enough_stock($total_quantity);
    }
}
