<?php

/**
 * Cart Helper
 *
 * @since 2.0.0
 * 
 */

namespace Bundler\Helpers;

use Bundler\Controllers\OfferController;

if (!defined('ABSPATH')) exit;

class Cart
{

    protected $offer_controller;

    public function __construct()
    {
        $this->offer_controller = new OfferController();

        // Before calculate totals for simple and variable products
        add_action('woocommerce_before_calculate_totals', array($this, 'before_calculate_totals'), 9999);

        // add_action('woocommerce_add_order_item_meta', array($this, 'add_order_item_meta'), 10, 2);

        // Hook filters
        add_filter('woocommerce_cart_item_price', array($this, 'cart_item_price_html'), 10, 3);
    }

    /**
     * @param mixed $cart_object
     * 
     * @return void
     */
    public function before_calculate_totals($cart_object)
    {

        if (is_admin() && !defined('DOING_AJAX'))
            return;

        if (!$cart_object) return;

        $cart_contents = $cart_object->cart_contents;
        if (!$cart_contents || empty($cart_contents) || !(is_array($cart_contents) || is_object($cart_contents))) return;

        $product_ids = $this->get_bundle_products_in_cart();
        if (!$product_ids || empty($product_ids)) return;

        foreach ($product_ids as $product_id) { // For each product (not item) in the cart

            $qty_in_cart = $cart_item_count = 0;

            foreach ($cart_contents as $key => $cart_item) {
                if ($cart_item['product_id'] == $product_id) {
                    if (isset($cart_item["_wbdl_data"])) {
                        $qty_in_cart += $cart_item['quantity']; // Get the product's quantity in cart
                        $cart_item_count++; // Get the number of lines for this product in the cart (could be 1 for simple product or > 1 for variable product if different variations in the cart)
                    }
                }
            }

            $discounted_total = $this->get_discounted_total($product_id, $qty_in_cart); // Calculate the total based on the bundles created for this product

            if ($discounted_total && ($discounted_total > 0) && ($qty_in_cart > 0)) {

                $price = ($discounted_total / $qty_in_cart);

                if ((strpos(strrev($price), '.') >= 3) || (strpos(strrev($price), ',') >= 3)) { // Woocommerce rounding issue will occur.

                    $price = number_format(round($price, 2), 2); // Round the unit price. This will be the price for all the items except the last one.

                    foreach ($cart_contents as $key => $cart_item) {
                        if ($cart_item['product_id'] == $product_id) {
                            if (isset($cart_item['_wbdl_data'])) {
                                $cart_item['_wbdl_data']['rounding'] = true;
                                if ($cart_item_count > 1) { // Not the last item in the cart
                                    $cart_item['_wbdl_data']['sale_price'] = $price;
                                    $cart_item['_wbdl_data']['multiline_price'] = false;
                                    $cart_item['data']->set_price($price);
                                } else {
                                    // Last item in the cart has a different price to make the total price match
                                    $last_item_price = ($discounted_total - ($price * ($qty_in_cart - $cart_item['quantity']))) / $cart_item['quantity'];
                                    $cart_item['_wbdl_data']['sale_price'] = $last_item_price;
                                    $cart_item['_wbdl_data']['multiline_price'] = ($cart_item['quantity'] > 1) ? 'multiple' : 'single'; // show multiple lines only if the total quantity is > 1
                                    $cart_item['data']->set_price($last_item_price);
                                }
                                $cart_item_count--;
                                WC()->cart->cart_contents[$key] = $cart_item;
                            }
                        }
                    }
                } else { // No rounding issue
                    foreach ($cart_contents as $key => $cart_item) {
                        if ($cart_item['product_id'] == $product_id) {
                            if (isset($cart_item['_wbdl_data'])) {
                                $cart_item['_wbdl_data']['sale_price'] = $price;
                                $cart_item['_wbdl_data']['multiline_price'] = false;
                                $cart_item['data']->set_price($price);
                            }
                        }
                        WC()->cart->cart_contents[$key] = $cart_item;
                    }
                }
            }
        }

        foreach ($cart_contents as $cart_item) {
            if (isset($cart_item['_wbdl_data']) && isset($cart_item['_wbdl_data']['sale_price'])) {
                $cart_item['data']->set_price($cart_item['_wbdl_data']['sale_price']);
            }
        }
    }

    /**
     * Calculate the discounted total based on the bundles prices
     * 
     * @param mixed $product_id
     * @param mixed $qty_in_cart
     * 
     * @return float $discounted_total
     */
    public function get_discounted_total($product_id, $qty_in_cart)
    {

        $discounted_total = 0;
        $available_bundles = $this->offer_controller->get_the_bundles_desc($product_id);

        if (!$available_bundles || empty($available_bundles) || $qty_in_cart == 0) return false;

        $i = $qty_in_cart;

        if (is_array($available_bundles)) {
            foreach ($available_bundles as $bundle) {
                if ($i >= $bundle->number_of_products) {

                    /* Note: it is possible to cumulate bundle discounts. The idea here is to start with the bigger bundle which is supposed to have the bigger discount, apply that discount and move to the next bundle.
                        Example: if we have 5 items in cart, bundle 1 = 3 items and bundle 2 = 2 items, we calculate the discounted price for the first 3 items using the first bundle because it has the biggest discount.*/
                    $max_bundles_nb = intval($i / $bundle->number_of_products);

                    if ($bundle->sale_price) {
                        $discounted_total += $this->get_wmc_price($bundle->sale_price) * $max_bundles_nb;
                    } else {
                        $discounted_total += $this->get_wmc_price($bundle->regular_price) * $max_bundles_nb;
                    }
                    $i -= $bundle->number_of_products * $max_bundles_nb;
                }
                if ($i == 0) break;
            }
        }

        return $discounted_total;
    }

    /**
     * Get multicurrency price in case a multicurrency plugin is enabled
     * 
     * @param mixed $price
     * @param mixed $wmc_price
     * @param string $is_wmc_custom_price
     * 
     * @return float $price
     */
    public function get_wmc_price($price)
    {
        if (has_filter("wbdl_get_wmc_price")) {
            $price = apply_filters("wbdl_get_wmc_price", $price);
        }

        return $price;
    }

    /**
     * Get IDs of bundle products in cart (doesn't include products with custom variations)
     * 
     * @return array
     */
    public function get_bundle_products_in_cart()
    {
        $product_ids = array();

        if (WC()->cart->cart_contents_count != 0) {
            foreach (WC()->cart->cart_contents as $cart_item_key => $cart_item) {
                if (isset($cart_item['_wbdl_data']) && isset($cart_item['_wbdl_data']['use_custom_variations']) && $cart_item['_wbdl_data']['use_custom_variations'] == 'off' && !in_array($cart_item['product_id'], $product_ids)) {
                    array_push($product_ids, $cart_item['product_id']);
                }
            }
        }
        return $product_ids;
    }

    /**
     * Edit the price html in the cart to show the price strikeout
     */
    public function cart_item_price_html($price, $cart_item, $cart_item_key)
    {
        if (is_null($cart_item['data'])) {
            return $price;
        }
        if (is_a($cart_item['data'], 'WC_Product')) {

            if (isset($cart_item['_wbdl_data'])) {
                if ($this->get_strikeout_price_html($cart_item) != "") {
                    $price = $this->get_strikeout_price_html($cart_item);
                }
            }
        }
        return $price;
    }

    /**
     * Build the Strikout Price HTML (to show on cart page) based on the product discount
     * 
     * @param mixed $cart_item
     * 
     * @return string
     */
    public function get_strikeout_price_html($cart_item)
    {
        $price_html = "";

        // Check if use custom variations is selected. This will have an impact on how to display prices (bundle price VS item price)
        $use_custom_variations = (isset($cart_item['_wbdl_data']) && $cart_item['_wbdl_data']['use_custom_variations'] != NULL) ? $cart_item['_wbdl_data']['use_custom_variations'] : 'off';

        if ($use_custom_variations == 'off') {

            // Get the regular & sale prices
            $regular_price = $cart_item['data']->get_regular_price();
            $sale_price = $cart_item['_wbdl_data']['sale_price'];

            // Check if the item shows multiline price
            $is_multiline = $cart_item['_wbdl_data']['multiline_price'];
            $is_rounding = $cart_item['_wbdl_data']['rounding'];
            $qty = $cart_item['quantity'] - 1;

            $price_html = '<div class="wbdl_cart_strikeout_line">';

            if ($regular_price && $sale_price && ($regular_price > $sale_price)) {
                $price_html .= '<del><span class="woocommerce-Price-amount amount" val="' . $regular_price . '"><bdi>' . wc_price($regular_price) . '</bdi></span></del>&nbsp;<ins><span class="woocommerce-Price-amount amount"><bdi>';
            } else {
                $price_html .= '<ins><span class="woocommerce-Price-amount amount"><bdi>';
            }

            // If multiline, the price has to be displayed in multiple lines to match the bundle price
            if ($is_rounding) {
                // Calculate the last item displayed price
                $last_item_subtotal = $sale_price * $cart_item['quantity'];
                $sale_price = number_format(round($sale_price, 2), 2);
                $last_item_adjusted_price = $last_item_subtotal - ($sale_price * ($cart_item['quantity'] - 1));
                if ($is_multiline) {
                    if ($is_multiline == 'multiple') {
                        $price_html .= wc_price($sale_price);
                        $price_html .= '</bdi></span></ins>&nbsp;x&nbsp;' . $qty . '</div>';
                        $price_html .= '<div class="wbdl_cart_strikeout_last_line"><del><span class="woocommerce-Price-amount amount"><bdi>';
                        $price_html .= wc_price($regular_price) . '</bdi></span></del>&nbsp;<ins><span class="woocommerce-Price-amount amount"><bdi>';
                        $price_html .= wc_price($last_item_adjusted_price);
                        $price_html .= '</bdi></span></ins>&nbsp;x&nbsp;1';
                    } else if ($is_multiline == 'single') {
                        $price_html .= wc_price($sale_price) . '</bdi></span></ins>';
                    }
                } else {
                    $price_html .= wc_price($sale_price) . '</bdi></span></ins>';
                }
            }
            // Else, the price has to be displayed in a single line because the price is the same for all items
            else {
                $price_html .= wc_price($sale_price) . '</bdi></span></ins>';
            }
            $price_html .= '</div>';
        }

        return $price_html;
    }
}
