<?php

/*
 * Class for creating tables into Database
 */

namespace Bundler\Models;

if (!defined('ABSPATH')) exit;

class DB
{

    // Tables name
    public static $offer_table_name = "wbdl_offer";
    public static $bundle_table_name = "wbdl_bundle";
    public static $settings_table_name = "wbdl_settings";

    public static $tables_count = 3;

    // current database version
    public static $current_db_version = '2.3';

    /**
     * Get the list of all custom tables starting with `wbdl_*`.
     *
     * @since 1.0.0
     *
     * @return array List of table names.
     */
    public static function get_existing_tables()
    {
        global $wpdb;

        $tables = $wpdb->get_results("SHOW TABLES LIKE '" . $wpdb->prefix . "wbdl_%'", 'ARRAY_N'); // phpcs:ignore

        return !empty($tables) ? wp_list_pluck($tables, 0) : [];
    }

    /**
     * Create tables
     */
    public static function create_tables()
    {
        global $wpdb;

        $installed_db_version = get_option('woobundles_db_version');
        $existing_tables = self::get_existing_tables();

        if (count($existing_tables) > 0) {
            if ($installed_db_version && ($installed_db_version != self::$current_db_version)) {
                if (floatval($installed_db_version) < 2.0) {
                    self::create_offer_table($wpdb);
                    self::db_update_2_0($wpdb);
                    self::create_bundle_table($wpdb);
                    self::create_settings_table($wpdb);
                    self::db_update_2_1($wpdb);
                    update_option('woobundles_db_version', self::$current_db_version);
                } else {
                    self::create_offer_table($wpdb);
                    self::create_bundle_table($wpdb);
                    self::create_settings_table($wpdb);
                    if (floatval($installed_db_version) < 2.1) self::db_update_2_1($wpdb);
                    update_option('woobundles_db_version', self::$current_db_version);
                }
            }
        } else {
            self::create_offer_table($wpdb);
            self::create_bundle_table($wpdb);
            self::create_settings_table($wpdb);
            if ($installed_db_version) {
                update_option('woobundles_db_version', self::$current_db_version);
            } else {
                add_option('woobundles_db_version', self::$current_db_version);
            }
            self::initialize_settings($wpdb);
        }
    }

    /**
     * Create offer table
     * 
     * @param mixed $wpdb
     * 
     * @return void
     */
    public static function create_offer_table($wpdb)
    {

        $charset_collate = $wpdb->get_charset_collate();

        $table_name = $wpdb->prefix . self::$offer_table_name;

        $sql = "CREATE TABLE $table_name ( 
            id mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
            title varchar(255) DEFAULT '' NOT NULL,
            product_id varchar(15) DEFAULT '' NOT NULL,
            type varchar(255) DEFAULT '' NOT NULL,
            status varchar(10) DEFAULT 'on' NOT NULL,
            priority int(10) DEFAULT 1 NOT NULL,
            wmc varchar(10) DEFAULT 'off',
            PRIMARY KEY  (id)) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    /**
     * Create bundles table
     * 
     * @param mixed $wpdb
     * 
     * @return void
     */
    public static function create_bundle_table($wpdb)
    {
        $charset_collate = $wpdb->get_charset_collate();

        $table_name = $wpdb->prefix . self::$bundle_table_name;

        $sql = "CREATE TABLE $table_name ( 
            id mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
            offer_id INT(11) DEFAULT NULL,
            bundle_title varchar(255) DEFAULT '' NOT NULL,
            image_url varchar(2048) DEFAULT '',
            number_of_products varchar(36) DEFAULT '1' NOT NULL,
            sale_price varchar(36) DEFAULT '' NOT NULL,
            regular_price varchar(36) DEFAULT '',
            wmc_sale_price varchar(255) DEFAULT '',
            wmc_regular_price varchar(255) DEFAULT '',
            use_custom_variations varchar(10) DEFAULT 'off',
            option1_name varchar(255) DEFAULT '',
            option1_value varchar(255) DEFAULT '',
            option2_name varchar(255) DEFAULT '',
            option2_value varchar(255) DEFAULT '',
            add_messages varchar(10) DEFAULT 'off',
            message varchar(255) DEFAULT '',
            discount_rule varchar(255) DEFAULT '',
            preselected_offer varchar(10) DEFAULT 'off',
            message_effect varchar(10) DEFAULT '',
            show_price_per_unit varchar(10) DEFAULT 'off',
            PRIMARY KEY  (id)) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    /**
     * create settings table
     * 
     * @param mixed $wpdb
     * 
     * @return void
     */
    public static function create_settings_table($wpdb)
    {

        $charset_collate = $wpdb->get_charset_collate();

        $table_name = $wpdb->prefix . self::$settings_table_name;

        $sql = "CREATE TABLE $table_name ( 
            `id` int(11) NOT NULL AUTO_INCREMENT,
            `design` varchar(20) DEFAULT 'classic',
            `design_mobile` varchar(20) DEFAULT 'classic',
            `bundles_title` varchar(220) DEFAULT NULL,
            `title_show` varchar(10) DEFAULT 'off',
            `qty_selector` varchar(10) DEFAULT 'off',
            `cart_redirect` varchar(10) DEFAULT 'off',
            `checkout_redirect` varchar(10) DEFAULT 'off',
            `atc_price_text` varchar(10) DEFAULT 'off',
            `background_color` varchar(20) DEFAULT NULL,
            `background_color_active` varchar(20) DEFAULT NULL,
            `border_color` varchar(20) DEFAULT NULL,
            `border_color_active` varchar(20) DEFAULT NULL,
            `title_color` varchar(20) DEFAULT NULL,
            `sale_price_color` varchar(20) DEFAULT NULL,
            `regular_price_color` varchar(20) DEFAULT NULL,
            `message1_color` varchar(20) DEFAULT NULL,
            `message2_color` varchar(20) DEFAULT NULL,
            PRIMARY KEY  (id)) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    /**
     * Initialise settings with default values
     * 
     * @param mixed $wpdb
     * 
     * @return void
     */
    public static function initialize_settings($wpdb)
    {
        $table_name = $wpdb->prefix . self::$settings_table_name;
        $wpdb->insert(
            $table_name,
            array(
                'id' => '1',
                'design' => 'design3',
                'design_mobile' => 'design3',
                'bundles_title' => 'Select your offer',
                'title_show' => 'off',
                'qty_selector' => 'off',
                'cart_redirect' => 'off',
                'checkout_redirect' => 'off',
                'atc_price_text' => 'off',
                'background_color' => '#ffffff00',
                'background_color_active' => '#fafbff',
                'border_color' => '#c6c6c6',
                'border_color_active' => '#565add',
                'title_color' => '#000000',
                'sale_price_color' => '#565add',
                'regular_price_color' => '#afafaf',
                'message1_color' => '#565add',
                'message2_color' => ''
            )
        );
    }

    /**
     * DB migration to the version 2.0
     * 
     * @param mixed $wpdb
     * 
     * @return void
     */
    public static function db_update_2_0($wpdb)
    {
        $bundles_table_old = $wpdb->prefix . 'wbdl_bundle_data';
        $settings_table_old = $wpdb->prefix . 'wbdl_bundle_settings';

        $bundles_table = $wpdb->prefix . self::$bundle_table_name;
        $settings_table = $wpdb->prefix . self::$settings_table_name;
        $offers_table = $wpdb->prefix . self::$offer_table_name;

        $wpdb->query('START TRANSACTION'); // Start a transaction

        try {
            // Step 0: Rename the tables (added from version 2.0)
            $query = "ALTER TABLE $bundles_table_old RENAME TO $bundles_table";
            $wpdb->query($query);
            $query = "ALTER TABLE $settings_table_old RENAME TO $settings_table";
            $wpdb->query($query);

            // Step 1: Add the offer_id column to the "Bundles" table
            $query = "ALTER TABLE $bundles_table ADD COLUMN offer_id INT(11) DEFAULT NULL AFTER id";
            $wpdb->query($query);

            // Step 2: Create offers in the "Offers" table for each bundle
            $query = "INSERT INTO $offers_table (title, product_id, type, status) SELECT CONCAT('Offer for product ', product_id), product_id, 'Bundle', 'on' FROM $bundles_table GROUP BY product_id";
            $wpdb->query($query);

            // Step 3: Set priority = offer id
            $query = "UPDATE $offers_table SET $offers_table.priority = $offers_table.id";
            $wpdb->query($query);

            // Step 4: Update the offer_id column in the "Bundles" table
            $query = "UPDATE $bundles_table INNER JOIN $offers_table ON $bundles_table.product_id = $offers_table.product_id SET $bundles_table.offer_id = $offers_table.id";
            $wpdb->query($query);

            // Step 5: Remove the product_id column from the "Bundles" table
            $query = "ALTER TABLE $bundles_table DROP COLUMN product_id";
            $wpdb->query($query);
        } catch (Exception $e) {
            $wpdb->query('ROLLBACK'); // Roll back the transaction if there's an error
        }
    }


    /**
     * DB migration to the version 2.1
     * 
     * @param mixed $wpdb
     * 
     * @return void
     */
    public static function db_update_2_1($wpdb)
    {
        $settings_table = $wpdb->prefix . self::$settings_table_name;

        $queries = array(
            "ALTER TABLE $settings_table MODIFY COLUMN design_mobile varchar(20) AFTER design;",
            "ALTER TABLE $settings_table MODIFY COLUMN background_color varchar(20) AFTER atc_price_text;",
            "ALTER TABLE $settings_table MODIFY COLUMN background_color_active varchar(20) AFTER background_color;",
            "ALTER TABLE $settings_table MODIFY COLUMN border_color varchar(20) AFTER background_color_active;",
            "ALTER TABLE $settings_table MODIFY COLUMN border_color_active varchar(20) AFTER border_color;",
            "ALTER TABLE $settings_table MODIFY COLUMN title_color varchar(20) AFTER border_color_active;",
            "ALTER TABLE $settings_table MODIFY COLUMN sale_price_color varchar(20) AFTER title_color;",
            "ALTER TABLE $settings_table MODIFY COLUMN regular_price_color varchar(20) AFTER sale_price_color;",
            "ALTER TABLE $settings_table MODIFY COLUMN message1_color varchar(20) AFTER regular_price_color;",
            "ALTER TABLE $settings_table MODIFY COLUMN message2_color varchar(20) AFTER message1_color;"
        );

        foreach ($queries as $query) {
            $wpdb->query($query);
        }
    }
}
