<?php

namespace Bundler\Views\Admin;

use Bundler\Controllers\OfferController;

if (!defined('ABSPATH')) exit;

if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class OffersTable extends \WP_List_Table
{
    protected $bulk_actions = array(
        'enable' => 'Enable',
        'disable' => 'Disable',
        'delete' => 'Delete',
    );

    protected $offer_controller;

    protected $per_page;

    // Constructor
    public function __construct()
    {
        $this->offer_controller = new OfferController();
        $this->per_page = $this->get_items_per_page('wbdl_offers_per_page', 20);
        parent::__construct(array(
            'singular' => 'offer',
            'plural' => 'offers',
            'ajax' => false
        ));
    }

    /**
     * Prepare table items
     */
    public function prepare_items()
    {
        $current_page = $this->get_pagenum(); // Get the current page number

        // Handle search
        $search = isset($_REQUEST['s']) ? sanitize_text_field($_REQUEST['s']) : '';

        $all_offers = $this->offer_controller->get_all_offers();

        // Apply search filter if a search query is provided
        if (!empty($search)) {
            $all_offers = array_filter($all_offers, function ($offer) use ($search) {
                return stripos($offer->title, $search) !== false;
            });
        }

        // Calculate the total number of items
        $total_items = count($all_offers);

        // Determine the offset
        $offset = ($current_page - 1) * $this->per_page;
        // Slice the offers array to get the subset for the current page
        $offers = array_slice($all_offers, $offset, $this->per_page);

        $columns = $this->get_columns();
        $sortable = $this->get_sortable_columns();

        $this->_column_headers = array($columns, array(), $sortable);
        $this->items = $offers;

        // Set the pagination arguments
        $this->set_pagination_args(array(
            'total_items' => $total_items, // Total number of items
            'per_page' => $this->per_page, // Number of items to display per page
            'total_pages' => ceil($total_items / $this->per_page), // Total number of pages
        ));
    }


    /**
     * Define table columns
     */
    public function get_columns()
    {
        $columns = array(
            'cb' => '<input type="checkbox">',
            //'offer_id' => 'Offer ID',
            'offer_title' => 'Title',
            'offer_priority' => 'Priority <span class="help-tip" title="For a given product, the latest offer that is enabled has the highest priority (priority 1 is the lowest).">?</span>',
            'product_id' => 'Product ID',
            'offer_type' => 'Discount Type',
            'offer_status' => 'Status',
            'action' => 'Action'
        );
        return $columns;
    }

    /**
     * Render table rows
     */
    public function display_rows()
    {
        foreach ($this->items as $offer) {
            $offer_type = '';

            switch ($offer->type) {
                case 'bundle':
                    $offer_type = esc_html('Bundle');
                    break;
                case 'qty_discount':
                    $offer_type = esc_html('Quantity Discount');
                    break;
                case 'bogo':
                    $offer_type = esc_html('Buy X Get X');
                    break;
            }

            $offer_status = $offer->status == "on" ? 'checked' : '';
            $product_id = $offer->product_id;
            $product = wc_get_product($product_id);

            echo '<tr id="offer-' . $offer->id . '" data-priority="' . $offer->id . '" class="wbdl-listing-offer-tr ui-sortable-handle">';
            echo '<th scope="row" class="check-column wbdl-listing-rule-check-box-align">';
            echo '<input id="cb-select-' . $offer->id . '" class="wbdl-rules-selector" type="checkbox" name="saved_offers[]" value="' . $offer->id . '">';
            echo '</th>';
            echo '<td class="title column-title has-row-actions column-primary page-title" data-colname="Title">';
            echo '<strong><a class="row-title" href="' . admin_url('admin.php?page=bundler&amp;path=edit-offer&amp;id=') . $offer->id . '">' . esc_html($offer->title) . '</a></strong>';
            echo '</td>';
            echo '<td class="author column-author" data-colname="Author"><abbr>' . esc_html($offer->priority) . '</abbr></td>';
            echo '<td class="author column-author" data-colname="ProductID"> <abbr>' . '#' . esc_html($product_id) . ' - ';
            if ($product) esc_html_e($product->get_name());
            else echo 'Product removed';
            echo '</abbr></td>';
            echo '<td class="author column-author" data-colname="Author"><abbr>' . $offer_type . '</abbr></td>';
            echo '<td class="date column-tag" data-colname="wdr-rule-status">';
            echo '<label class="switch switch-left-right">';
            echo '<input class="switch-input wbdl_manage_status" name="offer_status" type="checkbox" ' . $offer_status . '>';
            echo '<span class="switch-label" data-on="Active" data-off="Disabled"></span>';
            echo '<span class="switch-handle"></span>';
            echo '</label>';
            echo '<span class="wbdl-enabled-status">';
            echo '<span class="wbdl-listing-status-text"></span>';
            echo '</span>';
            echo '</td>';
            echo '<td class="wbdl-rule-buttons">';
            echo '<a target="_blank" href="' . get_permalink($offer->product_id) . '" class="wbdl-preview-offer" title="Preview"><i class="icofont-external-link"></i></a>';
            echo '<a class="" title="Edit" href="' . admin_url('admin.php?page=bundler&amp;path=edit-offer&amp;id=' . $offer->id) . '"><i class="icofont-ui-edit"></i></a>';
            echo '<a title="Delete"><i class="wbdl-delete-offer icofont-ui-delete"></i></a>';
            echo '</td>';
            echo '</tr>';
        }
    }

    /**
     * Get bulk actions
     * 
     * @return array
     */
    public function get_bulk_actions()
    {
        return $this->bulk_actions;
    }

    /**
     * Process bulk action
     */
    public function process_bulk_action()
    {
        $offer_ids = isset($_REQUEST['saved_offers']) ? $_REQUEST['saved_offers'] : array();
        // Verify the bulk action request
        switch ($this->current_action()) {
            case 'delete':
                foreach ($offer_ids as $offer_id) {
                    $this->offer_controller->deleteOffer($offer_id);
                }
                wp_redirect(admin_url('admin.php?page=bundler'));
                exit();
                break;

            case 'enable':
                foreach ($offer_ids as $offer_id) {
                    $this->offer_controller->updateOfferStatus($offer_id, 'on');
                }
                wp_redirect(admin_url('admin.php?page=bundler'));
                exit();
                break;

            case 'disable':
                foreach ($offer_ids as $offer_id) {
                    $this->offer_controller->updateOfferStatus($offer_id, 'off');
                }
                wp_redirect(admin_url('admin.php?page=bundler'));
                exit();
                break;
        }
    }

    /**
     * Define sortable columns
     * 
     * @return array
     */
    public function get_sortable_columns()
    {
        $sortable_columns = array(
            'offer_title' => array('offer_title', false),
            // 'offer_priority' => array('offer_priority', false),
        );
        return $sortable_columns;
    }

    /**
     * Define default column value when no column method is found
     * 
     * @param mixed $item
     * @param mixed $column_name
     * 
     * @return string
     */
    protected function column_default($item, $column_name)
    {
        return '';
    }

    // Display the table with form
    public function display()
    {
?>
        <form method="post" action="">
            <?php
            parent::display();
            ?>
        </form>
<?php
    }

    /**
     * Display the table navigation
     * 
     * @param mixed $which
     * 
     * @return void
     */
    public function display_tablenav($which)
    {
        // Display bulk actions
        echo '<div class="tablenav ' . $which . '">';
        if ($which === 'top') {
            // Display bulk actions section
            echo '<div class="alignleft actions">';
            $this->bulk_actions($which);
            echo '</div>';

            // Display items per page dropdown
            echo '<div class="alignleft actions">';
            echo '<label for="offers_per_page" class="screen-reader-text">Items Per Page</label>';
            echo '<select name="offers_per_page_' . $which . '" id="offers_per_page_' . $which . '">';
            $per_page_options = array(20, 50, 100); // Customize the options as needed
            foreach ($per_page_options as $option) {
                echo '<option value="' . $option . '"' . selected($this->per_page, $option, false) . '>' . $option . '</option>';
            }
            echo '</select>';
            submit_button('Apply', 'secondary', $which . '_per_page_action', false);
            echo '</div>';

            // Display search form
            echo '<div class="alignleft actions">';
            $this->search_box('Search Offers', 'offer_search');
            echo '</div>';


            // Check if the form is submitted to change the number of offers per page
            if (isset($_POST[$which . '_per_page_action']) && $_POST[$which . '_per_page_action'] === 'Apply') {
                // Update the user meta value with the selected per-page option
                $selected_per_page = $_POST['offers_per_page_' . $which];
                update_user_meta(get_current_user_id(), 'wbdl_offers_per_page', $selected_per_page);

                // Redirect to the same page to reflect the changes
                $redirect_url = admin_url('admin.php?page=bundler');
                wp_safe_redirect($redirect_url);
                exit();
            }
            // Display pagination
            $this->pagination($which);
        } else if ($which === 'bottom') {
            // Display pagination
            $this->pagination($which);
        }
        echo '<br class="clear">';
        echo '</div>'; // Close tablenav
    }
}
